#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var commander_1 = require("commander");
var nice_color_palettes_1 = __importDefault(require("nice-color-palettes"));
var chalk_1 = __importDefault(require("chalk"));
var make_matrix_1 = __importDefault(require("make-matrix"));
var mathUtils_1 = require("./mathUtils");
var cliUtils_1 = require("./cliUtils");
commander_1.program
    .version("1.1.0")
    .description("Displays an animated simplex noise field in your terminal")
    .option("-a, --angle", "outputs noise field as angles")
    .option("-b, --block", "outputs noise field as colour blocks")
    .option("-n, --number", "shows the numeric value of the noise field at each point")
    .option("-t, --timeout <milliseconds>", "adds a timeout (in millisecnods) to the noise field generation")
    .parse(process.argv);
var drawGridTimeout;
var startDrawLoop = function (pointPositions) {
    var nColours = 5;
    var randomPalette = mathUtils_1.shuffle(mathUtils_1.pickRandom(nice_color_palettes_1.default).slice(0, nColours));
    var color;
    var char = " ";
    var noiseZ = 0;
    var noiseZVel = 0.0000022;
    var zMult;
    var chalkWrite = function (color) {
        return commander_1.program.block ? chalk_1.default.bgHex(color) : chalk_1.default.hex(color);
    };
    var drawBlock = function (z) {
        var nChars = 5;
        zMult = Math.abs(Math.floor(z * nChars));
        var colourMap = Math.floor(mathUtils_1.mapRange(zMult, 0, nChars - 1, 0, nColours - 1));
        color = randomPalette[colourMap];
        return [color];
    };
    var drawAngle = function (z) {
        var nChars = 4;
        zMult = Math.abs(Math.floor(z * nChars));
        var colourMap = Math.floor(mathUtils_1.mapRange(zMult, 0, nChars - 1, 0, nColours - 1));
        color = randomPalette[colourMap];
        if (zMult === 0)
            char = "|";
        if (zMult === 1)
            char = "/";
        if (zMult === 2)
            char = "—";
        if (zMult === 3)
            char = "\\";
        return [color, char];
    };
    var drawAscii = function (z) {
        var nChars = 9;
        zMult = Math.abs(Math.floor(z * nChars));
        var colourMap = Math.floor(mathUtils_1.mapRange(zMult, 4, nChars - 1, 0, nColours - 1));
        color = randomPalette[colourMap];
        if ([0, 1, 2, 3].includes(zMult))
            char = " ";
        if (zMult === 4)
            char = ".";
        if (zMult === 5)
            char = "-";
        if (zMult === 6)
            char = "o";
        if (zMult === 7)
            char = "+";
        if (zMult === 8)
            char = "x";
        if (zMult === 9)
            char = "X";
        return [color, char];
    };
    var drawGrid;
    if (commander_1.program.block) {
        drawGrid = drawBlock;
    }
    else if (commander_1.program.angle) {
        drawGrid = drawAngle;
    }
    else {
        drawGrid = drawAscii;
    }
    drawGridTimeout = setInterval(function () {
        cliUtils_1.clearScrollback();
        pointPositions.forEach(function (position) {
            var y = position[0], x = position[1];
            var z = mathUtils_1.noise3D(x, y, noiseZ);
            drawGrid(z);
            if (commander_1.program.number)
                char = zMult;
            process.stdout.write(chalkWrite(color)(char));
            noiseZ += noiseZVel;
        });
    }, 80);
};
var createGrid = function () {
    var _a, _b;
    var columns = (_a = process.stdout.columns) !== null && _a !== void 0 ? _a : 0;
    var rows = (_b = process.stdout.rows) !== null && _b !== void 0 ? _b : 0;
    var pointPositions = make_matrix_1.default([rows, columns], function (pos) { return pos; }).flat(1);
    return pointPositions;
};
var init = function () {
    var grid = createGrid();
    startDrawLoop(grid);
    if (commander_1.program.timeout !== undefined) {
        setTimeout(function () {
            clearInterval(drawGridTimeout);
            cliUtils_1.clearScrollback();
        }, commander_1.program.timeout);
    }
};
init();
var initTimeout;
process.stdout.on("resize", function () {
    cliUtils_1.clearScrollback();
    clearInterval(drawGridTimeout);
    clearTimeout(initTimeout);
    initTimeout = setTimeout(function () { return init(); }, 200);
});
